@@ -1,11 +1,12 @@
 from copy import copy
-from dataclasses import dataclass
 from functools import wraps
 from time import time
-from typing import Optional, List, Any, Iterable, Dict, Sequence
+from typing import Optional, List, Any, Iterable, Dict, Sequence, Union
 import logging
 
 from .core.values.value_mapping import get_map_helper
+from .core.kb_sync.kb_iterator_config import KBIteratorConfig
+from .core.kb_sync.object_time_interval import ObjectTimeInterval
 from .tdm_builder.tdm_builder import AbstractTdmBuilder
 from .providers.gql_providers import AbstractGQLClient
 from .schema.api_schema import Query, Mutation, StoryPagination, ConceptLinkPagination, ConceptFactPagination, \
@@ -27,12 +28,6 @@ from .schema import utils_api_schema as uas
 from sgqlc.operation import Operation, Fragment
 
 
-@dataclass
-class KBIteratorConfig:
-    max_total_count: int
-    earliest_created_time: int
-
-
 logger = logging.getLogger(__name__)
 
 
@@ -805,6 +800,65 @@ class TalismanAPIAdapter:
         res = op + res  # type: Query
         return res.concept_link.pagination_concept_link_property.list_concept_property
 
+    def get_concept_time_intervals(
+        self, filter_settings: ConceptFilterSettings = None, max_interval_size: Optional[int] = None
+    ) -> Iterable[ObjectTimeInterval]:
+        if not filter_settings:
+            filter_settings = ConceptFilterSettings()
+        for time_interval in self._get_object_time_intervals(filter_settings, max_interval_size):
+            yield time_interval
+
+    def get_concept_link_time_intervals(
+        self, filter_settings: ConceptLinkFilterSettings = None, max_interval_size: Optional[int] = None
+    ) -> Iterable[ObjectTimeInterval]:
+        if not filter_settings:
+            filter_settings = ConceptLinkFilterSettings()
+        for time_interval in self._get_object_time_intervals(filter_settings, max_interval_size):
+            yield time_interval
+
+    def get_document_time_intervals(
+        self, filter_settings: DocumentFilterSettings = None, max_interval_size: Optional[int] = None
+    ) -> Iterable[ObjectTimeInterval]:
+        if not filter_settings:
+            filter_settings = DocumentFilterSettings()
+        for time_interval in self._get_object_time_intervals(filter_settings, max_interval_size):
+            yield time_interval
+
+    def _get_object_time_intervals(
+        self, filter_settings: Union[ConceptFilterSettings, ConceptLinkFilterSettings, DocumentFilterSettings],
+        max_interval_size: Optional[int] = None
+    ) -> Iterable[ObjectTimeInterval]:
+        max_interval_size = max_interval_size if max_interval_size else self.kb_iterator_config.max_total_count
+
+        creation_date_field_name = 'creation_date'
+        if isinstance(filter_settings, ConceptFilterSettings):
+            object_count = self.get_concept_count(filter_settings)
+        elif isinstance(filter_settings, ConceptLinkFilterSettings):
+            object_count = self.get_concept_link_count(filter_settings)
+        elif isinstance(filter_settings, DocumentFilterSettings):
+            object_count = self.get_documents_count(filter_settings)
+            creation_date_field_name = 'registration_date'
+        else:
+            raise Exception('Time division is only available for concepts, links and documents')
+        creation_date = getattr(filter_settings, creation_date_field_name, None)
+        start: int = getattr(creation_date, 'start', self.kb_iterator_config.earliest_created_time)
+        end: int = getattr(creation_date, 'end', int(time()))
+
+        if (object_count > max_interval_size) and (start < end):
+            middle = (end + start) // 2
+
+            for start, end in (start, middle), (middle + 1, end):
+                setattr(filter_settings, creation_date_field_name, TimestampInterval(start=start, end=end))
+                for time_interval in self._get_object_time_intervals(filter_settings, max_interval_size):
+                    yield time_interval
+        elif object_count > 0:
+            yield ObjectTimeInterval(
+                start_time=start,
+                end_time=end,
+                object_count=object_count,
+                max_interval_size=max_interval_size
+            )
+
     def create_concept(
         self, name: str, type_id: str, notes: str = None, with_properties=False, with_links=False,
         with_link_properties=False
